@@ -110,7 +110,7 @@ FILE * open_postpipe()
 }
 
 
-char printer_model[128] = "";
+char printer_model[256] = "";
 const char *accounting_prolog = NULL;
 char attrpath[256] = "";
 
@@ -141,11 +141,13 @@ dstr_t *jclappend;
  * WARNING: This logfile is a security hole; do not use in production. */
 int debug = 0;
 
-/* What path to use for filter programs and such. Your printer driver must be
- * in the path, as must be the renderer, $enscriptcommand, and possibly other
- * stuff. The default path is often fine on Linux, but may not be on other
- * systems. */
-char execpath [PATH_MAX] = "";
+/* Path to the GhostScript which foomatic-rip shall use */
+char gspath[PATH_MAX] = "gs";
+
+/* What 'echo' program to use.  It needs -e and -n.  Linux's builtin
+and regular echo work fine; non-GNU platforms may need to install
+gnu echo and put gecho here or something. */
+char echopath[PATH_MAX] = "echo";
 
 /* CUPS raster drivers are searched here */
 char cupsfilterpath[PATH_MAX] = "/usr/local/lib/cups/filter:"
@@ -159,14 +161,24 @@ void config_set_option(const char *key, const char *value)
 {
     if (strcmp(key, "debug") == 0)
         debug = atoi(value);
-    else if (strcmp(key, "execpath") == 0)
-        strlcpy(execpath, value, PATH_MAX);
+
+    /* What path to use for filter programs and such. Your printer driver must be
+     * in the path, as must be the renderer, $enscriptcommand, and possibly other
+     * stuff. The default path is often fine on Linux, but may not be on other
+     * systems. */
+    else if (strcmp(key, "execpath") == 0 && !isempty(value))
+        setenv("PATH", value, 1);
+
     else if (strcmp(key, "cupsfilterpath") == 0)
         strlcpy(cupsfilterpath, value, PATH_MAX);
     else if (strcmp(key, "preferred_shell") == 0)
         strlcpy(modern_shell, value, 32);
     else if (strcmp(key, "textfilter") == 0)
         set_fileconverter(value);
+    else if (strcmp(key, "gspath") == 0)
+        strlcpy(gspath, value, PATH_MAX);
+    else if (strcmp(key, "echo") == 0)
+        strlcpy(echopath, value, PATH_MAX);
 }
 
 void config_from_file(const char *filename)
@@ -179,11 +191,12 @@ void config_from_file(const char *filename)
     if (fh == NULL)
         return; /* no error here, only read config file if it is present */
 
-    while (fgets(line, 256, fh) != NULL) {
+    while (fgets(line, 256, fh) != NULL)
+    {
         key = strtok(line, " :\t\r\n");
         if (key == NULL || key[0] == '#')
             continue;
-        value = strtok(NULL, " :\t\r\n#");
+        value = strtok(NULL, " \t\r\n#");
         config_set_option(key, value);
     }
     fclose(fh);
@@ -272,15 +285,19 @@ char * extract_next_option(char *str, char **pagerange, char **key, char **value
 /* processes job->optstr */
 void process_cmdline_options()
 {
-    char *p, *nextopt, *pagerange, *key, *value;
+    char *p, *cmdlineopts, *nextopt, *pagerange, *key, *value;
     option_t *opt, *opt2;
     int optset;
     char tmp [256];
 
-    for (nextopt = extract_next_option(job->optstr->data, &pagerange, &key, &value);
+    _log("Printing system options:\n");
+    cmdlineopts = strdup(job->optstr->data);
+    for (nextopt = extract_next_option(cmdlineopts, &pagerange, &key, &value);
         key;
         nextopt = extract_next_option(nextopt, &pagerange, &key, &value))
     {
+        /* Consider only options which are not in the PPD file here */
+        if ((opt = find_option(key)) != NULL) continue;
         if (value)
             _log("Pondering option '%s=%s'\n", key, value);
         else
@@ -303,13 +320,6 @@ void process_cmdline_options()
         if (pagerange) {
             snprintf(tmp, 256, "pages:%s", pagerange);
             optset = optionset(tmp);
-
-            opt = find_option(key);
-            if (opt && (option_get_section(opt) != SECTION_ANYSETUP &&
-                        option_get_section(opt) != SECTION_PAGESETUP)) {
-                _log("This option (%s) is not a \"PageSetup\" or \"AnySetup\" option, so it cannot be restricted to a page range.\n", key);
-                continue;
-            }
         }
         else
             optset = optionset("userval");
@@ -388,38 +398,66 @@ void process_cmdline_options()
                     for Collate and StapleLocation?  These may be here...
                 */
             }
-            else {
-                /* Various non-standard printer-specific options */
-                if ((opt = find_option(key))) {
-                    if (!option_set_value(opt, optset, value)) {
-                        _log("  invalid choice \"%s\", using \"%s\" instead\n", 
-                                value, option_get_value(opt, optset));
-                    }
-                }
-                else if (spooler == SPOOLER_PPR_INT) {
-                    /* Unknown option, pass it to PPR's backend interface */
-                    if (!backendoptions)
-                        backendoptions = create_dstr();
-                    dstrcatf(backendoptions, "%s=%s ", key, value);
-                }
-                else
-                    _log("Unknown option %s=%s.\n", key, value);
-            }
+	    else if (spooler == SPOOLER_PPR_INT) {
+	        /* Unknown option, pass it to PPR's backend interface */
+	        if (!backendoptions)
+		    backendoptions = create_dstr();
+		dstrcatf(backendoptions, "%s=%s ", key, value);
+	    }
+	    else
+	        _log("Unknown option %s=%s.\n", key, value);
         }
         /* Custom paper size */
         else if ((opt = find_option("PageSize")) && option_set_value(opt, optset, key)) {
             /* do nothing, if the value could be set, it has been set */
         }
-        /* Standard bool args:
-           landscape; what to do here?
-           duplex; we should just handle this one OK now? */
-        else if (!prefixcasecmp(key, "no") && (opt = find_option(&key[2])))
-            option_set_value(opt, optset, "0");
-        else if ((opt = find_option(key)))
-            option_set_value(opt, optset, "1");
         else
             _log("Unknown boolean option \"%s\".\n", key);
     }
+    free(cmdlineopts);
+
+    _log("Options from the PPD file:\n");
+    cmdlineopts = strdup(job->optstr->data);
+    for (nextopt = extract_next_option(cmdlineopts, &pagerange, &key, &value);
+        key;
+        nextopt = extract_next_option(nextopt, &pagerange, &key, &value))
+    {
+        /* Consider only PPD file options here */
+        if ((opt = find_option(key)) == NULL) continue; 
+        if (value)
+            _log("Pondering option '%s=%s'\n", key, value);
+        else
+            _log("Pondering option '%s'\n", key);
+
+        if (pagerange) {
+            snprintf(tmp, 256, "pages:%s", pagerange);
+            optset = optionset(tmp);
+
+            if (opt && (option_get_section(opt) != SECTION_ANYSETUP &&
+                        option_get_section(opt) != SECTION_PAGESETUP)) {
+                _log("This option (%s) is not a \"PageSetup\" or \"AnySetup\" option, so it cannot be restricted to a page range.\n", key);
+                continue;
+            }
+        }
+        else
+            optset = optionset("userval");
+
+        if (value) {
+	    /* Various non-standard printer-specific options */
+	    if (!option_set_value(opt, optset, value)) {
+	        _log("  invalid choice \"%s\", using \"%s\" instead\n", 
+		     value, option_get_value(opt, optset));
+	    }
+        }
+        /* Standard bool args:
+           landscape; what to do here?
+           duplex; we should just handle this one OK now? */
+        else if (!prefixcasecmp(key, "no"))
+            option_set_value(opt, optset, "0");
+        else
+            option_set_value(opt, optset, "1");
+    }
+    free(cmdlineopts);
 }
 
 /* checks whether a pdq driver declaration file should be build
@@ -924,7 +962,7 @@ int print_file(const char *filename, int convert)
         }
     }
 
-    n = fread(buf, 1, sizeof(buf), file);
+    n = fread(buf, 1, sizeof(buf) - 1, file);
     buf[n] = '\0';
     type = guess_file_type(buf, n, &startpos);
     if (startpos > 0) {
@@ -1078,9 +1116,6 @@ int main(int argc, char** argv)
 
     config_from_file(CONFIG_PATH "/filter.conf");
 
-    if (!isempty(execpath))
-        setenv("PATH", execpath, 1);
-
     /* Command line options for verbosity */
     if (arglist_remove_flag(arglist, "-v"))
         verbose = 1;
