@@ -17,10 +17,13 @@
  */
 int test_gs_output_redirection()
 {
-    char * gstestcommand = GS_PATH " -dQUIET -dPARANOIDSAFER -dNOPAUSE -dBATCH -dNOMEDIAATTRS "
-        "-sDEVICE=pswrite -sstdout=%stderr -sOutputFile=/dev/null -c '(hello\n) print flush' 2>&1";
+    char gstestcommand[PATH_MAX];
     char output[10] = "";
 
+    snprintf(gstestcommand, PATH_MAX, "%s -dQUIET -dPARANOIDSAFER -dNOPAUSE "
+             "-dBATCH -dNOMEDIAATTRS -sDEVICE=pswrite -sstdout=%%stderr "
+             "-sOutputFile=/dev/null -c '(hello\n) print flush' 2>&1", gspath);
+
     FILE *pd = popen(gstestcommand, "r");
     if (!pd) {
         _log("Failed to execute ghostscript!\n");
@@ -45,55 +48,74 @@ void massage_gs_commandline(dstr_t *cmd)
 {
     int gswithoutputredirection = test_gs_output_redirection();
     size_t start, end;
+    dstr_t *gscmd, *cmdcopy;
 
     extract_command(&start, &end, cmd->data, "gs");
-
     if (start == end) /* cmd doesn't call ghostscript */
         return;
 
+    gscmd = create_dstr();
+    dstrncpy(gscmd, &cmd->data[start], end - start);
+
     /* If Ghostscript does not support redirecting the standard output
-       of the PostScript program to standard error with
-       '-sstdout=%stderr', sen the job output data to fd 3; errors
-       will be on 2(stderr) and job ps program interpreter output on
-       1(stdout). */
+       of the PostScript program to standard error with '-sstdout=%stderr', sen
+       the job output data to fd 3; errors will be on 2(stderr) and job ps
+       program interpreter output on 1(stdout). */
     if (gswithoutputredirection)
-        dstrreplace(cmd, "-sOutputFile=- ", "-sOutputFile=%stdout ", 0);
+        dstrreplace(gscmd, "-sOutputFile=- ", "-sOutputFile=%stdout ", 0);
     else
-        dstrreplace(cmd, "-sOutputFile=- ", "-sOutputFile=/dev/fd/3 ", 0);
+        dstrreplace(gscmd, "-sOutputFile=- ", "-sOutputFile=/dev/fd/3 ", 0);
 
     /* Use always buffered input. This works around a Ghostscript
-       bug which prevents printing encrypted PDF files with Adobe
-       Reader 8.1.1 and Ghostscript built as shared library
-       (Ghostscript bug #689577, Ubuntu bug #172264) */
-    dstrreplace(cmd, " - ", " -_ ", 0);
-    dstrreplace(cmd, " /dev/fd/0 ", " -_ ", 0);
+       bug which prevents printing encrypted PDF files with Adobe Reader 8.1.1
+       and Ghostscript built as shared library (Ghostscript bug #689577, Ubuntu
+       bug #172264) */
+    if (dstrendswith(gscmd, " -"))
+        dstrcat(gscmd, "_");
+    else
+        dstrreplace(gscmd, " - ", " -_ ", 0);
+    dstrreplace(gscmd, " /dev/fd/0", " -_ ", 0);
 
     /* Turn *off* -q (quiet!); now that stderr is useful! :) */
-    dstrreplace(cmd, " -q ", " ", 0);
+    dstrreplace(gscmd, " -q ", " ", 0);
 
     /* Escape any quotes, and then quote everything just to be sure...
-       Escaping a single quote inside single quotes is a bit complex as the shell
-       takes everything literal there. So we have to assemble it by concatinating
-       different quoted strings.
-       Finally we get e.g.: 'x'"'"'y' or ''"'"'xy' or 'xy'"'"'' or ... */
+       Escaping a single quote inside single quotes is a bit complex as the
+       shell takes everything literal there. So we have to assemble it by
+       concatinating different quoted strings.  Finally we get e.g.: 'x'"'"'y'
+       or ''"'"'xy' or 'xy'"'"'' or ... */
     /* dstrreplace(cmd, "'", "'\"'\"'"); TODO tbd */
 
-
-    dstrremove(cmd, start, 2);     /* Remove 'gs' */
+    dstrremove(gscmd, 0, 2);     /* Remove 'gs' */
     if (gswithoutputredirection)
-        dstrinsert(cmd, start, GS_PATH" -sstdout=%stderr ");
-    else {
-        dstrinsert(cmd, start, GS_PATH);
-        dstrinsert(cmd, end, " 3>&1 1>&2");
+    {
+        dstrprepend(gscmd, " -sstdout=%stderr ");
+        dstrprepend(gscmd, gspath);
     }
+    else
+    {
+        dstrprepend(gscmd, gspath);
+        dstrcat(gscmd, " 3>&1 1>&2");
+    }
+
+    /* put gscmd back into cmd, between 'start' and 'end' */
+    cmdcopy = create_dstr();
+    dstrcpy(cmdcopy, cmd->data);
+
+    dstrncpy(cmd, cmdcopy->data, start);
+    dstrcat(cmd, gscmd->data);
+    dstrcat(cmd, &cmdcopy->data[end]);
+
+    free_dstr(gscmd);
+    free_dstr(cmdcopy);
 
     /* If the renderer command line contains the "echo" command, replace the
      * "echo" by the user-chosen $myecho (important for non-GNU systems where
      * GNU echo is in a special path */
-    dstrreplace(cmd, "echo", ECHO, 0); /* TODO search for \wecho\w */
+    dstrreplace(cmd, "echo", echopath, 0); /* TODO search for \wecho\w */
 }
 
-char * read_line(FILE *stream)
+char * read_line(FILE *stream, size_t *readbytes)
 {
     char *line;
     size_t alloc = 64, len = 0;
@@ -106,21 +128,32 @@ char * read_line(FILE *stream)
             alloc *= 2;
             line = realloc(line, alloc);
         }
-        if (c == '\n')
-            break;
         line[len] = (char)c;
         len++;
+        if (c == '\n')
+            break;
     }
 
     line[len] = '\0';
+    *readbytes = len;
     return line;
 }
 
+void write_binary_data(FILE *stream, const char *data, size_t bytes)
+{
+    int i;
+    for (i=0; i < bytes; i++)
+    {
+	fputc(data[i], stream);
+    }
+}
+
 /*
  * Read all lines containing 'jclstr' from 'stream' (actually, one more) and
  * return them in a zero terminated array.
  */
-static char ** read_jcl_lines(FILE *stream, const char *jclstr)
+static char ** read_jcl_lines(FILE *stream, const char *jclstr,
+			      size_t *readbinarybytes)
 {
     char *line;
     char **result;
@@ -129,19 +162,22 @@ static char ** read_jcl_lines(FILE *stream, const char *jclstr)
     result = malloc(alloc * sizeof(char *));
 
     /* read from the renderer output until the first non-JCL line appears */
-    while ((line = read_line(stream)))
+    while ((line = read_line(stream, readbinarybytes)))
     {
         if (cnt >= alloc -1)
         {
             alloc *= 2;
-            result = realloc(result, alloc);
+            result = realloc(result, alloc * sizeof(char *));
         }
         result[cnt] = line;
-        cnt++;
         if (!strstr(line, jclstr))
             break;
+        /* Remove newline from the end of a line containing JCL */
+        result[cnt][*readbinarybytes - 1] = '\0';
+        cnt++;
     }
 
+    cnt++;
     result[cnt] = NULL;
     return result;
 }
@@ -153,16 +189,21 @@ static int jcl_keywords_equal(const char *jclline1, const char *jclline2,
 
     j1 = strstr(jclline1, jclstr);
     if (!j1) return 0;
-    p1 = strchrnul(skip_whitespace(j1), '=') - 1;
-    while (isspace(*p1))
+    if (!(p1 = strchr(skip_whitespace(j1), '=')))
+        p1 = j1 + strlen(j1);
+    p1--;
+    while (p1 > j1 && isspace(*p1))
         p1--;
 
     j2 = strstr(jclline2, jclstr);
     if (!j2) return 0;
-    p2 = strchrnul(skip_whitespace(j2), '=') - 1;
-    while (isspace(*p2))
+    if (!(p2 = strchr(skip_whitespace(j2), '=')))
+        p2 = j2 + strlen(j2);
+    p2--;
+    while (p2 > j2 && isspace(*p2))
         p2--;
 
+    if (p1 - j1 != p2 - j2) return 0;
     return strncmp(j1, j2, p1 - j1 + 1) == 0;
 }
 
@@ -202,6 +243,7 @@ static void argv_write(FILE *stream, char **argv, const char *sep)
 static int write_merged_jcl_options(FILE *stream,
                                     char **original_opts,
                                     char **opts,
+                                    size_t readbinarybytes,
                                     const char *jclstr)
 {
     char *p = strstr(original_opts[0], jclstr);
@@ -213,7 +255,7 @@ static int write_merged_jcl_options(FILE *stream,
     {
         fprintf(stream, "%s", jclbegin);
         argv_write(stream, opts, "\n");
-        fprintf(stream, "%s\n", original_opts[0]);
+        write_binary_data(stream, original_opts[0], readbinarybytes);
         return 0;
     }
 
@@ -233,7 +275,7 @@ static int write_merged_jcl_options(FILE *stream,
         if (p)
             fprintf(stream, "%s\n", p);
 
-        fprintf(stream, "%s\n", original_opts[1]);
+        write_binary_data(stream, original_opts[1], readbinarybytes);
         return 1;
     }
 
@@ -246,13 +288,14 @@ static int write_merged_jcl_options(FILE *stream,
         if (!jcl_options_find_keyword(original_opts, *optsp, jclstr))
 	    fprintf(stream, "%s\n", *optsp);
 
-    for (optsp = original_opts; *optsp; optsp++) {
+    for (optsp = original_opts; *(optsp + 1); optsp++) {
         if (optsp != original_opts) p = *optsp;
         if (jcl_options_find_keyword(opts, p, jclstr))
 	  fprintf(stream, "%s\n", jcl_options_find_keyword(opts, p, jclstr));
 	else
             fprintf(stream, "%s\n", p);
     }
+    write_binary_data(stream, *optsp, readbinarybytes);
 
     return 1;
 }
@@ -273,6 +316,7 @@ int exec_kid4(FILE *in, FILE *out, void *user_arg)
 {
     FILE *fileh = open_postpipe();
     int driverjcl;
+    size_t readbinarybytes;
 
     log_jcl();
 
@@ -283,13 +327,20 @@ int exec_kid4(FILE *in, FILE *out, void *user_arg)
     {
         if (!isspace(jclprepend[0][0]))
         {
-            char *jclstr = strndup(jclprepend[0],
-                                   strcspn(jclprepend[0], " \t\n\r"));
-            char **jclheader = read_jcl_lines(in, jclstr);
+            char *jclstr, **jclheader;
+            size_t pos;
+
+            pos = strcspn(jclprepend[0], " \t\n\r");
+            jclstr = malloc(pos +1);
+            strncpy(jclstr, jclprepend[0], pos);
+            jclstr[pos] = '\0';
+
+            jclheader = read_jcl_lines(in, jclstr, &readbinarybytes);
 
             driverjcl = write_merged_jcl_options(fileh,
                                                  jclheader,
                                                  jclprepend,
+                                                 readbinarybytes,
                                                  jclstr);
 
             free(jclstr);
